@@ -10,7 +10,7 @@ import numpy as np
 
 from ..util.meshgrid import meshgrid_nd
 from ..util.functions import FreezableClass, link_or_copy
-from ..util.logger import logger
+from astropy import log as logger
 from .grid_helpers import single_grid_dims
 
 
@@ -158,7 +158,7 @@ class AMRGrid(FreezableClass):
                         n_dust_q, shape_q = single_grid_dims(grid.quantities[quantity])
                         if n_dust is None:
                             n_dust = n_dust_q
-                        else:
+                        elif n_dust_q is not None:
                             if n_dust != n_dust_q:
                                 raise ValueError("Not all dust lists in the grid have the same size")
             return n_dust
