@@ -9,12 +9,10 @@ import numpy as np
 from ..util.constants import c, pi
 from ..util.functions import FreezableClass
 from ..dust import SphericalDust
-from ..util.logger import logger
+from astropy import log as logger
 from ..util.decorator import decorator
 from ..grid import CartesianGrid, SphericalPolarGrid, CylindricalPolarGrid, OctreeGrid, AMRGrid
 
-from .helpers import find_last_iteration
-
 STOKESD = {}
 STOKESD['I'] = 0
 STOKESD['Q'] = 1
@@ -223,13 +221,6 @@ class ModelOutput(FreezableClass):
             array. This is only returned if uncertainties were requested.
         '''
 
-        # Set default units
-        if units is None:
-            if distance is None:
-                units = 'ergs/s'
-            else:
-                units = 'ergs/cm^2/s'
-
         # Check argument types
         if type(stokes) is not str:
             raise ValueError("stokes argument should be a string")
@@ -323,6 +314,13 @@ class ModelOutput(FreezableClass):
         if inside_observer and distance is not None:
             raise ValueError("Cannot specify distance for inside observers")
 
+        # Set default units
+        if units is None:
+            if distance is None and not inside_observer:
+                units = 'ergs/s'
+            else:
+                units = 'ergs/cm^2/s'
+
         # Optionally scale by distance
         if distance is not None or inside_observer:
 
@@ -336,7 +334,7 @@ class ModelOutput(FreezableClass):
             elif units == 'mJy':
                 scale = 1.e26 / nu
             else:
-                raise Exception("Unknown units: %s" % units)
+                raise ValueError("Unknown units: %s" % units)
 
             # Scale by distance
             if distance:
@@ -440,10 +438,20 @@ class ModelOutput(FreezableClass):
         else:
             raise ValueError("Unknown Stokes parameter: %s" % stokes)
 
-        if uncertainties:
-            return wav, flux, unc
-        else:
-            return wav, flux
+        from .sed import SED
+        sed = SED(nu=nu, val=flux, unc=unc if uncertainties else None, units=units)
+
+        # Add aperture information
+        sed.ap_min = g['seds'].attrs['apmin']
+        sed.ap_max = g['seds'].attrs['apmax']
+
+        # Add distance
+        sed.distance = distance
+
+        # Save whether the SED was from an inside observer
+        sed.inside_observer = inside_observer
+
+        return sed
 
     @on_the_fly_hdf5
     def get_image(self, stokes='I', group=0, technique='peeled',
@@ -548,13 +556,6 @@ class ModelOutput(FreezableClass):
             array. This is only returned if uncertainties were requested.
         '''
 
-        # Set default units
-        if units is None:
-            if distance is None:
-                units = 'ergs/s'
-            else:
-                units = 'ergs/cm^2/s'
-
         # Check argument types
         if type(stokes) is not str:
             raise ValueError("stokes argument should be a string")
@@ -648,6 +649,76 @@ class ModelOutput(FreezableClass):
         if inside_observer and distance is not None:
             raise ValueError("Cannot specify distance for inside observers")
 
+        # Set default units
+        if units is None:
+            if distance is None and not inside_observer:
+                units = 'ergs/s'
+            else:
+                units = 'ergs/cm^2/s'
+
+        # Find pixel dimensions of image
+        ny, nx = flux.shape[-3:-1]
+
+        # Find physical and angular extent, and pixel area in steradians
+        if inside_observer:
+
+            # Physical extent cannot be set
+            x_min = x_max = y_min = y_max = None
+
+            # Find extent of the image
+            lon_min = g['images'].attrs['xmin']
+            lon_max = g['images'].attrs['xmax']
+            lat_min = g['images'].attrs['ymin']
+            lat_max = g['images'].attrs['ymax']
+
+            # Need to construct a mesh since every pixel might have a
+            # different size
+            lon = np.linspace(np.radians(lon_min), np.radians(lon_max), nx + 1)
+            lat = np.cos(np.linspace(np.radians(90. - lat_min), np.radians(90. - lat_max), ny + 1))
+            dlon = lon[1:] - lon[:-1]
+            dlat = lat[:-1] - lat[1:]
+            DLON, DLAT = np.meshgrid(dlon, dlat)
+
+            # Find pixel area in steradians
+            pix_area_sr = DLON * DLAT
+
+        else:
+
+            # Find physical extent of the image
+            x_min = g['images'].attrs['xmin']
+            x_max = g['images'].attrs['xmax']
+            y_min = g['images'].attrs['ymin']
+            y_max = g['images'].attrs['ymax']
+
+            if distance is not None:
+
+                # Find angular extent
+                lon_min_rad = np.arctan(x_min / distance)
+                lon_max_rad = np.arctan(x_max / distance)
+                lat_min_rad = np.arctan(y_min / distance)
+                lat_max_rad = np.arctan(y_max / distance)
+
+                # Find pixel size in arcseconds
+                pix_dx = abs(lon_max_rad - lon_min_rad) / float(nx)
+                pix_dy = abs(lat_max_rad - lat_min_rad) / float(ny)
+
+                # Find pixel area in steradians
+                pix_area_sr = pix_dx * pix_dy
+
+                # Find angular extent in degrees
+                lon_min = np.degrees(lon_min_rad)
+                lon_max = np.degrees(lon_max_rad)
+                lat_min = np.degrees(lat_min_rad)
+                lat_max = np.degrees(lat_max_rad)
+
+            else:
+
+                # Angular extent cannot be set
+                lon_min = lon_max = lat_min = lat_max = None
+
+                # Pixel area in steradians cannot be set
+                pix_area_sr = None
+
         # Optionally scale by distance
         if distance is not None or inside_observer:
 
@@ -661,41 +732,12 @@ class ModelOutput(FreezableClass):
             elif units == 'mJy':
                 scale = 1.e26 / nu
             elif units == 'MJy/sr':
-
-                # Find spatial extent of the image
-                xmin = g['images'].attrs['xmin']
-                xmax = g['images'].attrs['xmax']
-                ymin = g['images'].attrs['ymin']
-                ymax = g['images'].attrs['ymax']
-
-                # Find pixel dimensions of image
-                ny, nx = flux.shape[-3:-1]
-
-                # Find pixel resolution in radians/pixel
                 if inside_observer:
-
-                    # Need to construct a mesh since every pixel might have a different size
-                    x = np.linspace(np.radians(xmin), np.radians(xmax), nx + 1)
-                    y = np.cos(np.linspace(np.radians(90. - ymin), np.radians(90. - ymax), ny + 1))
-                    dx = x[1:] - x[:-1]
-                    dy = y[:-1] - y[1:]
-                    DX, DY = np.meshgrid(dx, dy)
-                    pix_area = DX * DY
-
-                    scale = 1.e17 / nu[np.newaxis, np.newaxis, :] / pix_area[:, :, np.newaxis]
-
+                    scale = 1.e17 / nu[np.newaxis, np.newaxis, :] / pix_area_sr[:, :, np.newaxis]
                 else:
-
-                    pix_dx = abs(np.arctan((xmax - xmin) / float(nx) / distance))
-                    pix_dy = abs(np.arctan((ymax - ymin) / float(ny) / distance))
-
-                    # Find pixel area in steradians
-                    pix_area = pix_dx * pix_dy
-
-                    scale = 1.e17 / nu / pix_area
-
+                    scale = 1.e17 / nu / pix_area_sr
             else:
-                raise Exception("Unknown units: %s" % units)
+                raise ValueError("Unknown units: %s" % units)
 
             # Scale by distance
             if distance:
@@ -708,7 +750,8 @@ class ModelOutput(FreezableClass):
 
             scale = np.repeat(1., len(nu))
 
-        # If in 32-bit mode, need to convert to 64-bit because of scaling/polarization to be safe
+        # If in 32-bit mode, need to convert to 64-bit because of
+        # scaling/polarization to be safe
         if flux.dtype == np.float32:
             flux = flux.astype(np.float64)
         if uncertainties and unc.dtype == np.float32:
@@ -787,10 +830,31 @@ class ModelOutput(FreezableClass):
         else:
             raise ValueError("Unknown Stokes parameter: %s" % stokes)
 
-        if uncertainties:
-            return wav, flux, unc
-        else:
-            return wav, flux
+        from .image import Image
+        image = Image(nu=nu, val=flux, unc=unc if uncertainties else None, units=units)
+
+        # Add physical extent
+        image.x_min = x_min
+        image.x_max = x_max
+        image.y_min = y_min
+        image.y_max = y_max
+
+        # Add angular extent
+        image.lon_min = lon_min
+        image.lon_max = lon_max
+        image.lat_min = lat_min
+        image.lat_max = lat_max
+
+        # Add pixel area in steradians
+        image.pix_area_sr = pix_area_sr
+
+        # Add distance
+        image.distance = distance
+
+        # Save whether the image was from an inside observer
+        image.inside_observer = inside_observer
+
+        return image
 
     @on_the_fly_hdf5
     def get_available_components(self, iteration=-1):
@@ -804,6 +868,8 @@ class ModelOutput(FreezableClass):
             The iteration to retrieve the grid for. The default is to return the components for the last iteration
         '''
 
+        from .helpers import find_last_iteration
+
         # If iteration is last one, find iteration number
         if iteration == -1:
             iteration = find_last_iteration(self.file)
@@ -831,6 +897,9 @@ class ModelOutput(FreezableClass):
             An object containing information about the geometry and quantities
         '''
 
+        from .helpers import find_last_iteration
+
+
         if self.file['Input'].file != self.file.file:
             # Workaround for h5py bug - can't access link directly,
             # need to use file attribute
@@ -938,6 +1007,8 @@ class ModelOutput(FreezableClass):
         At the moment, this method only works on regular grids, not AMR or Oct-tree grids
         '''
 
+        from .helpers import find_last_iteration
+
         warnings.warn("get_physical_grid is deprecated, use get_quantities instead", DeprecationWarning)
 
         # Check that dust_id was not specified if grid is n_photons
