@@ -2,10 +2,10 @@ import numpy as np
 
 from .. import Model
 
-from .test_helpers import get_test_model_noimaging, random_filename
+from .test_helpers import get_test_model_noimaging, random_id
 
 
-def test_monochromatic_wav():
+def test_monochromatic_wav(tmpdir):
 
     model = Model()
     model.set_cartesian_grid([-1., 1.], [-1., 1.], [-1., 1.])
@@ -19,11 +19,11 @@ def test_monochromatic_wav():
 
     model.set_n_photons(initial=1, imaging_sources=1, imaging_dust=1)
 
-    model.write(random_filename())
-    model.run(random_filename())
+    model.write(tmpdir.join(random_id()).strpath)
+    model.run(tmpdir.join(random_id()).strpath)
 
 
-def test_model_spectrum():
+def test_model_spectrum(tmpdir):
 
     model = get_test_model_noimaging()
 
@@ -32,5 +32,5 @@ def test_model_spectrum():
     source.spectrum = (np.array([1.e5, 1.e15]),
                        np.array([1., 1.]))
 
-    model.write(random_filename())
-    model.run(random_filename())
+    model.write(tmpdir.join(random_id()).strpath)
+    model.run(tmpdir.join(random_id()).strpath)
