@@ -1,15 +1,35 @@
 from __future__ import print_function, division
 
-import atpy
 import numpy as np
+from astropy.table import Table, Column
 
 from ..grid.amr_grid import AMRGridView
 
 from ..util.functions import B_nu, random_id, FreezableClass, \
-                             is_numpy_array, bool2str, monotonically_increasing
+    is_numpy_array, bool2str, str2bool, monotonically_increasing
 from ..util.integrate import integrate_loglog
 from ..util.validator import validate_scalar
-from ..util.logger import logger
+from astropy import log as logger
+
+
+def read_source(handle):
+    source_type = handle.attrs['type'].decode('ascii')
+    if source_type == 'spot':
+        return SpotSource.read(handle)
+    elif source_type == 'point':
+        return PointSource.read(handle)
+    elif source_type == 'sphere':
+        return SphericalSource.read(handle)
+    elif source_type == 'extern_sph':
+        return ExternalSphericalSource.read(handle)
+    elif source_type == 'extern_box':
+        return ExternalBoxSource.read(handle)
+    elif source_type == 'map':
+        return MapSource.read(handle)
+    elif source_type == 'plane_parallel':
+        return PlaneParallelSource.read(handle)
+    else:
+        raise ValueError("Unexpected source type: {0}".format(source_type))
 
 
 class Source(FreezableClass):
@@ -80,7 +100,11 @@ class Source(FreezableClass):
     @property
     def spectrum(self):
         '''
-        The spectrum of the source, specified either as an atpy.Table instance with ``'nu'`` and ``'fnu'`` columns, or as a ``(nu, fnu)`` tuple, where the frequency is given in Hz, and the flux is given as F_nu (units are unimportant since the normalization is set by the luminosity).
+        The spectrum of the source, specified either as an astropy.table.Table
+        instance with ``'nu'`` and ``'fnu'`` columns, or as a ``(nu, fnu)``
+        tuple, where the frequency is given in Hz, and the flux is given as
+        F_nu (units are unimportant since the normalization is set by the
+        luminosity).
         '''
         return self._spectrum
 
@@ -92,14 +116,14 @@ class Source(FreezableClass):
             if hasattr(self, '_temperature') and self._temperature is not None:
                 raise Exception("A temperature has already been set, so cannot set a spectrum")
 
-            if isinstance(value, atpy.Table):
+            if isinstance(value, Table):
 
                 if 'nu' not in value.columns:
-                    raise TypeError("spectrum ATpy Table does not contain a"
+                    raise TypeError("spectrum Table does not contain a"
                                     " 'nu' column")
 
                 if 'fnu' not in value.columns:
-                    raise TypeError("spectrum ATpy Table does not contain an"
+                    raise TypeError("spectrum Table does not contain an"
                                     " 'fnu' column")
 
                 nu, fnu = value['nu'], value['fnu']
@@ -134,13 +158,30 @@ class Source(FreezableClass):
             else:
 
                 raise TypeError('spectrum should be specified either as an '
-                                'atpy.Table instance, or a tuple of two 1-D'
-                                'Numpy arrays (nu, fnu) with the same length')
+                                'astropy.table.Table instance, or a tuple '
+                                'of two 1-D Numpy arrays (nu, fnu) with the '
+                                'same length')
 
             # Check if frequency array has duplicate values
             if len(np.unique(nu)) != len(nu):
                 raise ValueError("nu sequence contains duplicate values")
 
+            # Check for any negative values
+
+            if np.any(nu <= 0.):
+                raise ValueError("nu should be strictly positive")
+
+            if np.any(fnu < 0.):
+                raise ValueError("fnu should be positive")
+
+            # Check for any NaN or Inf values
+
+            if np.any(np.isnan(nu) | np.isinf(nu)):
+                raise ValueError("nu contains NaN/Inf values")
+
+            if np.any(np.isnan(fnu) | np.isinf(fnu)):
+                raise ValueError("fnu contains NaN/Inf values")
+
             # Check if spectrum needs sorting
             if not monotonically_increasing(nu):
                 logger.warn("Spectrum is being re-sorted in order of increasing frequency")
@@ -178,6 +219,26 @@ class Source(FreezableClass):
         norm = integrate_loglog(nu, fnu)
         return nu, fnu / norm * self.luminosity
 
+    @classmethod
+    def read(cls, handle):
+
+        self = cls()
+
+        self.luminosity = handle.attrs['luminosity']
+
+        self.peeloff = str2bool(handle.attrs['peeloff'])
+
+        if handle.attrs['spectrum'] == b'spectrum':
+            self.spectrum = Table(np.array(handle['spectrum']))
+        elif handle.attrs['spectrum'] == b'temperature':
+            self.temperature = handle.attrs['temperature']
+        elif handle.attrs['spectrum'] == b'lte':
+            pass
+        else:
+            raise ValueError('Unexpected value for `spectrum`: %s' % handle.attrs['spectrum'])
+
+        return self
+
     def write(self, handle):
 
         self._check_all_set()
@@ -188,10 +249,10 @@ class Source(FreezableClass):
 
         if self.spectrum is not None:
             handle.attrs['spectrum'] = np.string_('spectrum'.encode('utf-8'))
-            table = atpy.Table(name='spectrum')
-            table.add_column('nu', self.spectrum['nu'])
-            table.add_column('fnu', self.spectrum['fnu'])
-            table.write(handle, type='hdf5')
+            table = Table()
+            table.add_column(Column(data=self.spectrum['nu'], name='nu'))
+            table.add_column(Column(data=self.spectrum['fnu'], name='fnu'))
+            table.write(handle, path='spectrum')
         elif self.temperature is not None:
             handle.attrs['spectrum'] = np.string_('temperature'.encode('utf-8'))
             handle.attrs['temperature'] = self.temperature
@@ -221,9 +282,9 @@ class SpotSource(Source):
         ----------
         luminosity : float
             The luminosity of the source (in ergs/s)
-        spectrum : atpy.Table or tuple
+        spectrum : astropy.table.Table or tuple
             The spectrum of the source, specified either as:
-                * an ATpy table with ``nu`` and ``fnu`` column
+                * an Astropy Table with ``nu`` and ``fnu`` column
                 * a ``(nu, fnu)`` tuple
             ``nu`` should be in Hz, and the units for ``fnu`` are not
             important, since the luminosity determined the absolute scaling``
@@ -254,6 +315,20 @@ class SpotSource(Source):
         if self.has_lte_spectrum():
             raise ValueError("Spot source cannot have LTE spectrum")
 
+    @classmethod
+    def read(cls, handle):
+
+        if not handle.attrs['type'] == b'spot':
+            raise ValueError("Source is not a SpotSource")
+
+        self = super(SpotSource, cls).read(handle)
+
+        self.longitude = handle.attrs['longitude']
+        self.latitude = handle.attrs['latitude']
+        self.radius = handle.attrs['radius']
+
+        return self
+
     def write(self, handle, name):
         self._check_all_set()
         g = handle.create_group(name)
@@ -326,6 +401,14 @@ class PointSource(Source):
         if self.has_lte_spectrum():
             raise ValueError("Point source cannot have LTE spectrum")
 
+    @classmethod
+    def read(cls, handle):
+        if not handle.attrs['type'] == b'point':
+            raise ValueError("Source is not a PointSource")
+        self = super(PointSource, cls).read(handle)
+        self.position = (handle.attrs['x'], handle.attrs['y'], handle.attrs['z'])
+        return self
+
     def write(self, handle, name):
         self._check_all_set()
         g = handle.create_group(name)
@@ -421,6 +504,22 @@ class SphericalSource(Source):
         if self.has_lte_spectrum():
             raise ValueError("Spherical source cannot have LTE spectrum")
 
+    @classmethod
+    def read(cls, handle):
+        if not handle.attrs['type'] == b'sphere':
+            raise ValueError("Source is not a SphericalSource")
+        self = super(SphericalSource, cls).read(handle)
+        self.position = (handle.attrs['x'], handle.attrs['y'], handle.attrs['z'])
+        self.radius = handle.attrs['r']
+        self.limb = str2bool(handle.attrs['limb'])
+
+        # Read in spots
+        for group in handle:
+            if 'Spot' in group:
+                self._spots.append(SpotSource.read(handle[group]))
+
+        return self
+
     def write(self, handle, name):
 
         self._check_all_set()
@@ -444,7 +543,9 @@ class SphericalSource(Source):
         All arguments are passed to :class:`~hyperion.sources.SpotSource`,
         so see that class for more details
         '''
-        self._spots.append(SpotSource(*args, **kwargs))
+        spot = SpotSource(*args, **kwargs)
+        self._spots.append(spot)
+        return spot
 
 
 class ExternalSphericalSource(Source):
@@ -518,6 +619,15 @@ class ExternalSphericalSource(Source):
         if self.has_lte_spectrum():
             raise ValueError("External spherical source cannot have LTE spectrum")
 
+    @classmethod
+    def read(cls, handle):
+        if not handle.attrs['type'] == b'extern_sph':
+            raise ValueError("Source is not a ExternalSphericalSource")
+        self = super(ExternalSphericalSource, cls).read(handle)
+        self.position = (handle.attrs['x'], handle.attrs['y'], handle.attrs['z'])
+        self.radius = handle.attrs['r']
+        return self
+
     def write(self, handle, name):
 
         self._check_all_set()
@@ -586,6 +696,16 @@ class ExternalBoxSource(Source):
         if self.has_lte_spectrum():
             raise ValueError("External spherical source cannot have LTE spectrum")
 
+    @classmethod
+    def read(cls, handle):
+        if not handle.attrs['type'] == b'extern_box':
+            raise ValueError("Source is not a ExternalBoxSource")
+        self = super(ExternalBoxSource, cls).read(handle)
+        self.bounds = [(handle.attrs['xmin'], handle.attrs['xmax']),
+                       (handle.attrs['ymin'], handle.attrs['ymax']),
+                       (handle.attrs['zmin'], handle.attrs['zmax'])]
+        return self
+
     def write(self, handle, name):
 
         self._check_all_set()
@@ -647,6 +767,14 @@ class MapSource(Source):
         if is_numpy_array(self.map) and np.all(self.map == 0.):
             raise ValueError("map is zero everywhere")
 
+    @classmethod
+    def read(cls, handle):
+        if not handle.attrs['type'] == b'map':
+            raise ValueError("Source is not a MapSource")
+        self = super(MapSource, cls).read(handle)
+        self.map = np.array(handle['Luminosity map'])
+        return self
+
     def write(self, handle, name, grid, compression=True, map_dtype=float):
 
         self._check_all_set()
@@ -654,8 +782,8 @@ class MapSource(Source):
         g = handle.create_group(name)
         g.attrs['type'] = np.string_('map'.encode('utf-8'))
         grid.write_single_array(g, "Luminosity map", self.map,
-                                  compression=compression,
-                                  physics_dtype=map_dtype)
+                                compression=compression,
+                                physics_dtype=map_dtype)
         Source.write(self, g)
 
 
@@ -757,6 +885,16 @@ class PlaneParallelSource(Source):
         if self.has_lte_spectrum():
             raise ValueError("Point source cannot have LTE spectrum")
 
+    @classmethod
+    def read(cls, handle):
+        if not handle.attrs['type'] == b'plane_parallel':
+            raise ValueError("Source is not a PlaneParallelSource")
+        self = super(PlaneParallelSource, cls).read(handle)
+        self.position = (handle.attrs['x'], handle.attrs['y'], handle.attrs['z'])
+        self.radius = handle.attrs['r']
+        self.direction = (handle.attrs['theta'], handle.attrs['phi'])
+        return self
+
     def write(self, handle, name):
         self._check_all_set()
         g = handle.create_group(name)
