@@ -1,12 +1,8 @@
 from __future__ import print_function, division
 
-import string
-import random
-
-import h5py
-import atpy
-import pytest
+from astropy.tests.helper import pytest
 import numpy as np
+from astropy.table import Table, Column
 
 from .. import Source, PointSource, SpotSource, SphericalSource, ExternalSphericalSource, ExternalBoxSource, MapSource, PlaneParallelSource
 from ...grid import CartesianGrid, \
@@ -14,17 +10,11 @@ from ...grid import CartesianGrid, \
                     SphericalPolarGrid, \
                     AMRGrid, \
                     OctreeGrid
+from ...util.functions import virtual_file
 
 ALL_SOURCES = [Source, PointSource, SpotSource, SphericalSource, ExternalSphericalSource, ExternalBoxSource, MapSource, PlaneParallelSource]
 
 
-def random_id(length=32):
-    return ''.join(random.sample(string.ascii_letters + string.digits, length))
-
-
-def virtual_file():
-    return h5py.File(random_id(), driver='core', backing_store=False)
-
 # LUMINOSITY
 
 
@@ -88,21 +78,21 @@ def test_temperature_invalid4(source_type):
 
 
 @pytest.mark.parametrize(('source_type'), ALL_SOURCES)
-def test_spectrum_atpy(source_type):
-    t = atpy.Table()
-    t.add_column('nu', [1, 2, 3])
-    t.add_column('fnu', [1, 2, 3])
+def test_spectrum_astropy(source_type):
+    t = Table()
+    t.add_column(Column(data=[1, 2, 3], name='nu'))
+    t.add_column(Column(data=[1, 2, 3], name='fnu'))
     s = source_type()
     s.spectrum = t
 
 
 @pytest.mark.parametrize(('source_type'), ALL_SOURCES)
-def test_spectrum_atpy_invalid(source_type):
-    t = atpy.Table()  # table is empty, so invalid
+def test_spectrum_astropy_invalid(source_type):
+    t = Table()  # table is empty, so invalid
     s = source_type()
     with pytest.raises(TypeError) as exc:
         s.spectrum = t
-    assert exc.value.args[0] == 'spectrum ATpy Table does not contain a \'nu\' column'
+    assert exc.value.args[0] == 'spectrum Table does not contain a \'nu\' column'
 
 
 @pytest.mark.parametrize(('source_type'), ALL_SOURCES)
@@ -197,6 +187,40 @@ def test_set_spectrum_temperature(source_type):
         s.temperature = 1000.  # spectrum has already been specified
     assert exc.value.args[0] == 'A spectrum has already been set, so cannot set a temperature'
 
+@pytest.mark.parametrize(('source_type'), ALL_SOURCES)
+def test_spectrum_negative(source_type):
+
+    nu = np.array([1, 2, -3])
+    fnu = np.array([1, 2, 3])
+    s = source_type()
+    with pytest.raises(ValueError) as exc:
+        s.spectrum = (nu, fnu)
+    assert exc.value.args[0] == 'nu should be strictly positive'
+
+    nu = np.array([1, 2, 3])
+    fnu = np.array([1, -2, 3])
+    s = source_type()
+    with pytest.raises(ValueError) as exc:
+        s.spectrum = (nu, fnu)
+    assert exc.value.args[0] == 'fnu should be positive'
+
+@pytest.mark.parametrize(('source_type'), ALL_SOURCES)
+def test_spectrum_nan(source_type):
+
+    nu = np.array([1, 2, np.nan])
+    fnu = np.array([1, 2, 3])
+    s = source_type()
+    with pytest.raises(ValueError) as exc:
+        s.spectrum = (nu, fnu)
+    assert exc.value.args[0] == 'nu contains NaN/Inf values'
+
+    nu = np.array([1, 2, 3])
+    fnu = np.array([1, np.inf, 3])
+    s = source_type()
+    with pytest.raises(ValueError) as exc:
+        s.spectrum = (nu, fnu)
+    assert exc.value.args[0] == 'fnu contains NaN/Inf values'
+
 # POSITION
 
 SOURCES_POSITION = [PointSource,
